@@ -6,11 +6,12 @@ Parallel::ForkManager - A simple parallel processing fork manager
 
   use Parallel::ForkManager;
 
-  $pm = Parallel::ForkManager->new($MAX_PROCESSES);
+  my $pm = Parallel::ForkManager->new($MAX_PROCESSES);
 
-  foreach $data (@all_data) {
+  DATA_LOOP:
+  foreach my $data (@all_data) {
     # Forks and returns the pid for the child:
-    my $pid = $pm->start and next;
+    my $pid = $pm->start and next DATA_LOOP;
 
     ... do some work with $data in the child process ...
 
@@ -30,7 +31,7 @@ The code for a downloader would look something like this:
 
   ...
 
-  @links=(
+  my @links=(
     ["http://www.foo.bar/rulez.data","rulez_data.txt"],
     ["http://new.host/more_data.doc","more_data.doc"],
     ...
@@ -41,12 +42,13 @@ The code for a downloader would look something like this:
   # Max 30 processes for parallel download
   my $pm = Parallel::ForkManager->new(30);
 
+  LINKS:
   foreach my $linkarray (@links) {
-    $pm->start and next; # do the fork
+    $pm->start and next LINKS; # do the fork
 
-    my ($link,$fn) = @$linkarray;
+    my ($link, $fn) = @$linkarray;
     warn "Cannot get $fn from $link"
-      if getstore($link,$fn) != RC_OK;
+      if getstore($link, $fn) != RC_OK;
 
     $pm->finish; # do the exit in the child process
   }
@@ -75,7 +77,7 @@ C for child.
 
 =over 5
 
-=item new $processes [, $tempdir]  # P
+=item new $processes
 
 Instantiate a new Parallel::ForkManager object. You must specify the maximum
 number of children to fork off. If you specify 0 (zero), then no children
@@ -83,13 +85,12 @@ will be forked. This is intended for debugging purposes.
 
 The optional second parameter, $tempdir, is only used if you want the
 children to send back a reference to some data (see RETRIEVING DATASTRUCTURES
-below). If not provided, it is set to $L<File::Temp::tempdir().
+below). If not provided, it is set via a call to L<File::Temp>::tempdir().
 
 The new method will die if the temporary directory does not exist or it is not
-a directory, whether you provided this parameter or the
-$L<File::Temp::tempdir() is used.
+a directory.
 
-=item start [ $process_identifier ]  # P
+=item start [ $process_identifier ]
 
 This method does the fork. It returns the pid of the child process for
 the parent, and 0 for the child process. If the $processes parameter
@@ -100,7 +101,7 @@ An optional $process_identifier can be provided to this method... It is used by
 the "run_on_finish" callback (see CALLBACKS) for identifying the finished
 process.
 
-=item finish [ $exit_code [, $data_structure_reference] ]  # C
+=item finish [ $exit_code [, $data_structure_reference] ]
 
 Closes the child process by exiting and accepts an optional exit code
 (default exit code is 0) which can be retrieved in the parent via callback.
@@ -111,15 +112,36 @@ it's contents back to the parent. If you use the program in debug mode
 If the $data_structure_reference is provided, then it is serialized and
 passed to the parent process. See RETRIEVING DATASTRUCTURES for more info.
 
-=item set_max_procs $processes  # P
+=item set_max_procs $processes
 
 Allows you to set a new maximum number of children to maintain.
 
-=item wait_all_children  # P
+=item wait_all_children
 
 You can call this method to wait for all the processes which have been
 forked. This is a blocking wait.
 
+=item max_procs 
+
+Returns the maximal number of processes the object will fork.
+
+=item running_procs
+
+Returns the pids of the forked processes currently monitored by the
+C<Parallel::ForkManager>. Note that children are still reported as running
+until the fork manager harvest them, via the next call to
+C<start> or C<wait_all_children>.
+
+    my @pids = $pm->running_procs;
+
+    my $nbr_children =- $pm->running_procs;
+
+=item wait_for_available_procs( $n )
+
+Wait until C<$n> available process slots are available.
+If C<$n> is not given, defaults to I<1>.
+
+
 =back
 
 =head1 CALLBACKS
@@ -131,12 +153,12 @@ The callbacks can be defined with the following methods:
 
 =over 4
 
-=item run_on_finish $code [, $pid ]  # P
+=item run_on_finish $code [, $pid ]
 
 You can define a subroutine which is called when a child is terminated. It is
 called in the parent process.
 
-The paremeters of the $code are the following:
+The parameters of the $code are the following:
 
   - pid of the process, which is terminated
   - exit code of the program
@@ -145,7 +167,7 @@ The paremeters of the $code are the following:
   - core dump (1 if there was core dump at exit)
   - datastructure reference or undef (see RETRIEVING DATASTRUCTURES)
 
-=item run_on_start $code  # P
+=item run_on_start $code
 
 You can define a subroutine which is called when a child is started. It called
 after the successful startup of a child in the parent process.
@@ -155,13 +177,13 @@ The parameters of the $code are the following:
   - pid of the process which has been started
   - identification of the process (if provided in the "start" method)
 
-=item run_on_wait $code, [$period]  # P
+=item run_on_wait $code, [$period]
 
 You can define a subroutine which is called when the child process needs to wait
 for the startup. If $period is not defined, then one call is done per
 child. If $period is defined, then $code is called periodically and the
-module waits for $period seconds betwen the two calls. Note, $period can be
-fractional number also. The exact "$period seconds" is not guarranteed,
+module waits for $period seconds between the two calls. Note, $period can be
+fractional number also. The exact "$period seconds" is not guaranteed,
 signals can shorten and the process scheduler can make it longer (on busy
 systems).
 
@@ -210,16 +232,19 @@ This small example can be used to get URLs in parallel.
 
   use Parallel::ForkManager;
   use LWP::Simple;
-  my $pm= Parallel::ForkManager->new(10);
+
+  my $pm = Parallel::ForkManager->new(10);
+
+  LINKS:
   for my $link (@ARGV) {
-    $pm->start and next;
-    my ($fn)= $link =~ /^.*\/(.*?)$/;
+    $pm->start and next LINKS;
+    my ($fn) = $link =~ /^.*\/(.*?)$/;
     if (!$fn) {
       warn "Cannot determine filename from $fn\n";
     } else {
-      $0.=" ".$fn;
+      $0 .= " " . $fn;
       print "Getting $fn from $link\n";
-      my $rc=getstore($link,$fn);
+      my $rc = getstore($link, $fn);
       print "$link downloaded. response code: $rc\n";
     };
     $pm->finish;
@@ -247,7 +272,7 @@ Example of a program using callbacks to get child exit codes:
   });
 
   $pm->run_on_start( sub {
-      my ($pid,$ident)=@_;
+      my ($pid, $ident)=@_;
       print "** $ident started, pid: $pid\n";
   });
 
@@ -257,8 +282,9 @@ Example of a program using callbacks to get child exit codes:
     0.5
   );
 
+  NAMES:
   foreach my $child ( 0 .. $#names ) {
-    my $pid = $pm->start($names[$child]) and next;
+    my $pid = $pm->start($names[$child]) and next NAMES;
 
     # This code is the child process
     print "This is $names[$child], Child number $child\n";
@@ -278,36 +304,37 @@ In this simple example, each child sends back a string reference.
 
   use Parallel::ForkManager 0.7.6;
   use strict;
-  
+
   my $pm = Parallel::ForkManager->new(2, '/server/path/to/temp/dir/');
-  
+
   # data structure retrieval and handling
   $pm -> run_on_finish ( # called BEFORE the first call to start()
     sub {
       my ($pid, $exit_code, $ident, $exit_signal, $core_dump, $data_structure_reference) = @_;
-  
+
       # retrieve data structure from child
       if (defined($data_structure_reference)) {  # children are not forced to send anything
         my $string = ${$data_structure_reference};  # child passed a string reference
         print "$string\n";
-      } else {  # problems occuring during storage or retrieval will throw a warning
+      }
+      else {  # problems occuring during storage or retrieval will throw a warning
         print qq|No message received from child process $pid!\n|;
       }
     }
   );
-  
+
   # prep random statement components
   my @foods = ('chocolate', 'ice cream', 'peanut butter', 'pickles', 'pizza', 'bacon', 'pancakes', 'spaghetti', 'cookies');
   my @preferences = ('loves', q|can't stand|, 'always wants more', 'will walk 100 miles for', 'only eats', 'would starve rather than eat');
-  
+
   # run the parallel processes
-  my $person = '';
-  foreach $person (qw(Fred Wilma Ernie Bert Lucy Ethel Curly Moe Larry)) {
-    $pm->start() and next;
-  
+  PERSONS:
+  foreach my $person (qw(Fred Wilma Ernie Bert Lucy Ethel Curly Moe Larry)) {
+    $pm->start() and next PERSONS;
+
     # generate a random statement about food preferences
     my $statement = $person . ' ' . $preferences[int(rand @preferences)] . ' ' . $foods[int(rand @foods)];
-  
+
     # send it back to the parent process
     $pm->finish(0, \$statement);  # note that it's a scalar REFERENCE, not the scalar itself
   }
@@ -322,15 +349,15 @@ process whatever is retrieved.
   use Parallel::ForkManager 0.7.6;
   use Data::Dumper;  # to display the data structures retrieved.
   use strict;
-  
+
   my $pm = Parallel::ForkManager->new(20);  # using the system temp dir $L<File::Temp::tempdir()
-  
+
   # data structure retrieval and handling
   my %retrieved_responses = ();  # for collecting responses
   $pm -> run_on_finish (
     sub {
       my ($pid, $exit_code, $ident, $exit_signal, $core_dump, $data_structure_reference) = @_;
-  
+
       # see what the child sent us, if anything
       if (defined($data_structure_reference)) {  # test rather than assume child sent anything
         my $reftype = ref($data_structure_reference);
@@ -338,15 +365,15 @@ process whatever is retrieved.
         if (1) {  # simple on/off switch to display the contents
           print &Dumper($data_structure_reference) . qq|end of "$ident" sent structure\n\n|;
         }
-        
+
         # we can also collect retrieved data structures for processing after all children have exited
         $retrieved_responses{$ident} = $data_structure_reference;
       } else {
-        print qq|ident "$ident" did not send anything.\n\n|;  
+        print qq|ident "$ident" did not send anything.\n\n|;
       }
     }
   );
-  
+
   # generate a list of instructions
   my @instructions = (  # a unique identifier and what the child process should send
     {'name' => '%ENV keys as a string', 'send' => 'keys'},
@@ -356,23 +383,23 @@ process whatever is retrieved.
     {'name' => 'Invalid send instructions', 'send' => 'Na Na Nana Na'},
     {'name' => 'ENV values in an array', 'send' => 'values'},
   );
-  
-  my $instruction = '';
-  foreach $instruction (@instructions) {
-    $pm->start($instruction->{'name'}) and next;  # this time we are using an explicit, unique child process identifier
-  
+
+  INSTRUCTS:
+  foreach my $instruction (@instructions) {
+    $pm->start($instruction->{'name'}) and next INSTRUCTS;  # this time we are using an explicit, unique child process identifier
+
     # last step in child processing
     $pm->finish(0) unless $instruction->{'send'};  # no data structure is sent unless this child is told what to send.
-    
+
     if ($instruction->{'send'} eq 'keys') {
       $pm->finish(0, \join(', ', keys %ENV));
-      
+
     } elsif ($instruction->{'send'} eq 'values') {
       $pm->finish(0, [values %ENV]);  # kinda useless without knowing which keys they belong to...
-      
+
     } elsif ($instruction->{'send'} eq 'all') {
       $pm->finish(0, \%ENV);  # remember, we are not "returning" anything, just copying the hash to disc
-    
+
     # demonstrate clearly that the child determines what type of reference to send
     } elsif ($instruction->{'send'} eq 'random') {
       my $string = q|I'm just a string.|;
@@ -382,14 +409,14 @@ process whatever is retrieved.
       $pm->finish(0, \$string) if ($return_choice eq 'string');
       $pm->finish(0, \@array) if ($return_choice eq 'array');
       $pm->finish(0, \%hash) if ($return_choice eq 'hash');
-      
+
     # as a responsible child, inform parent that their instruction was invalid
-    } else {  
+    } else {
       $pm->finish(0, \qq|Invalid instructions: "$instruction->{'send'}".|);  # ordinarily I wouldn't include invalid input in a response...
     }
   }
   $pm->wait_all_children;  # blocks until all forked processes have exited
-  
+
   # post fork processing of returned data structures
   for (sort keys %retrieved_responses) {
     print qq|Post processing "$_"...\n|;
@@ -412,17 +439,21 @@ processes, although I don't think it makes sense.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2000-2010 Szab, Balzs (dLux)
+=encoding utf8
+
+Copyright (c) 2000-2010 Szabó, Balázs (dLux)
 
 All right reserved. This program is free software; you can redistribute it
 and/or modify it under the same terms as Perl itself.
 
 =head1 AUTHOR
 
-  dLux (Szab, Balzs) <dlux@dlux.hu>
+  dLux (Szabó, Balázs) <dlux@dlux.hu>
 
 =head1 CREDITS
 
+  Gábor Szabó (szabgab@cpan.org)  (co-maintainer)
+  Michael Gang (bug report)
   Noah Robin <sitz@onastick.net> (documentation tweaks)
   Chuck Hirstius <chirstius@megapathdsl.net> (callback exit status, example)
   Grant Hopwood <hopwoodg@valero.com> (win32 port)
@@ -437,27 +468,35 @@ use Storable qw(store retrieve);
 use File::Spec;
 use File::Temp ();
 use File::Path ();
+use Carp;
+
 use strict;
 use vars qw($VERSION);
-$VERSION="1.0.0";
+$VERSION="1.10_1";
 $VERSION = eval $VERSION;
 
 sub new {
-  my ($c,$processes, $tempdir)=@_;
+  my ($c,$processes,$tempdir)=@_;
 
   my $h={
     max_proc   => $processes,
     processes  => {},
     in_child   => 0,
     parent_pid => $$,
+    auto_cleanup => ($tempdir ? 0 : 1),
   };
-  
+
+
   # determine temporary directory for storing data structures
   # add it to Parallel::ForkManager object so children can use it
   # We don't let it clean up so it won't do it in the child process
   # but we have our own DESTROY to do that.
-  $h->{tempdir} = File::Temp::tempdir(CLEANUP => 0);
-  
+  if (not defined($tempdir) or not length($tempdir)) {
+    $tempdir = File::Temp::tempdir(CLEANUP => 0);
+  }
+  die qq|Temporary directory "$tempdir" doesn't exist or is not a directory.| unless (-e $tempdir && -d _);  # ensure temp dir exists and is indeed a directory
+  $h->{tempdir} = $tempdir;
+
   return bless($h,ref($c)||$c);
 };
 
@@ -542,7 +581,7 @@ sub wait_one_child {
       if (not $retrieved or $@) {
         warn(qq|The storable module was unable to retrieve the child's data structure from the temporary file "$storable_tempfile":  | . join(', ', $@));
       }
-      
+
       # clean up after ourselves
       unlink $storable_tempfile;
     }
@@ -564,6 +603,25 @@ sub wait_all_children {
 
 *wait_all_childs=*wait_all_children; # compatibility;
 
+sub max_procs { $_[0]->{max_proc}; }
+
+sub running_procs {
+    my $self = shift;
+
+    my @pids = keys %{ $self->{processes} };
+    return @pids;
+}
+
+sub wait_for_available_procs {
+    my( $self, $nbr ) = @_;
+    $nbr ||= 1;
+
+    croak "nbr processes '$nbr' higher than the max nbr of processes (@{[ $self->max_procs ]})"
+        if $nbr > $self->max_procs;
+
+    $self->wait_one_child until $self->max_procs - $self->running_procs >= $nbr;
+}
+
 sub run_on_finish {
   my ($s,$code,$pid)=@_;
 
@@ -614,44 +672,45 @@ sub set_max_procs {
   $s->{max_proc} = $mp;
 }
 
-# OS dependant code follows...
-
 sub _waitpid { # Call waitpid() in the standard Unix fashion.
-  return waitpid($_[1],$_[2]);
+    my( $self, $pid, $flag ) = @_;
+
+    return $flag ? $self->_waitpid_non_blocking : $self->_waitpid_blocking;
 }
 
-# On ActiveState Perl 5.6/Win32 build 625, waitpid(-1, &WNOHANG) always
-# blocks unless an actual PID other than -1 is given.
-sub _NT_waitpid {
-  my ($s, $pid, $par) = @_;
+sub _waitpid_non_blocking {
+    my $self = shift;
 
-  if ($par == &WNOHANG) { # Need to nonblock on each of our PIDs in the pool.
-    my @pids = keys %{ $s->{processes} };
-    # Simulate -1 (no processes awaiting cleanup.)
-    return -1 unless scalar(@pids);
-    # Check each PID in the pool.
-    my $kid;
-    foreach $pid (@pids) {
-      $kid = waitpid($pid, $par);
-      return $kid if $kid != 0; # AS 5.6/Win32 returns negative PIDs.
+    for my $pid ( $self->running_procs ) {
+        my $p = waitpid $pid, &WNOHANG or next;
+        if ( $p == -1 ) {
+            warn "child process '$pid' disappeared. A call to `waitpid` outside of Parallel::ForkManager might have reaped it.\n";
+            # it's gone. let's clean the process entry
+            delete $self->{processes}{$pid};
+        }
+        else {
+            return $pid;
+        }
     }
-    return $kid;
-  } else { # Normal waitpid() call.
-    return waitpid($pid, $par);
-  }
+
+    return 0;
 }
 
-{
-  local $^W = 0;
-  if ($^O eq 'NT' or $^O eq 'MSWin32') {
-    *_waitpid = \&_NT_waitpid;
-  }
+sub _waitpid_blocking {
+    my $self = shift;
+
+    while() {
+        my $pid = $self->_waitpid_non_blocking;
+        return $pid if $pid;
+
+        sleep 1;
+    }
 }
 
 sub DESTROY {
   my ($self) = @_;
 
-  if ($self->{parent_pid} == $$ && -d $self->{tempdir}) {
+  if ($self->{auto_cleanup} && $self->{parent_pid} == $$ && -d $self->{tempdir}) {
     File::Path::remove_tree($self->{tempdir});
   }
 }
